#include <Arduino.h>
#include <mrm-8x8a.h>
#include <mrm-board.h>
#include <mrm-bldc2x50.h>
#include <mrm-bldc4x2.5.h>
#include <mrm-can-bus.h>
#include <mrm-col-b.h>
#include <mrm-col-can.h>
#include <mrm-common.h>
#include <mrm-fet-can.h>
#include <mrm-imu.h>
// #include <mrm-ir-finder2.h>
#include <mrm-ir-finder3.h>
// #include <mrm-ir-finder-can.h>
#include <mrm-lid-can-b.h>
#include <mrm-lid-can-b2.h>
#include <mrm-mot2x50.h>
#include <mrm-mot4x10.h>
#include <mrm-mot4x3.6can.h>
#include <mrm-node.h>
#include <mrm-pid.h>
#include <mrm-ref-can.h>
#include <mrm-robot.h>
#include "mrm-robot-line.h"
#include "mrm-robot-maze.h"
#include "mrm-robot-min.h"
#include "mrm-robot-soccer.h"
#include <mrm-servo.h>
#include <mrm-switch.h>
#include <mrm-therm-b-can.h>
// #include <mrm-us.h>
#include <mrm-us-b.h>
#include <mrm-us1.h>

Robot *robot;

#define MIN_MOTOR_SPEED(speed_modifier) (((speed_modifier) >= 0 ? 1 : -1) * MIN_SPEED_THRESHOLD)
#define MAX_MOTOR_SPEED(speed_modifier) (((speed_modifier) >= 0 ? 1 : -1) * MAX_SPEED_THRESHOLD)

#define SPEED_STEP 10
#define SPEED_BEND_BOOST 1.2
#define TILT_MODIFIER 2
#define MIN_SPEED_THRESHOLD 30
#define MAX_SPEED_THRESHOLD 100
#define TIGHT_BEND_BOOST 10

#define ANGLE_STRAIGHT 0
#define ANGLE_RIGHT 41
#define ANGLE_LEFT -ANGLE_RIGHT
#define ANGLE_BACKWARDS 180
#define ANGLE_RIGHT_BACKWARDS -131
#define ANGLE_LEFT_BACKWARDS 131
#define STRAFE 0
#define ROTATE_LEFT 20
#define ROTATE_RIGHT -20
#define DIRECTION_FORWARDS 1
#define DIRECTION_BACKWARDS -1

bool init_private = false;
int speed_modifier = 0;

int leftmotor = speed_modifier;
int rightmotor = speed_modifier;
int last_direction = DIRECTION_FORWARDS;
int angle_private = 0;
int rotation_private = 0;

char msg_private[2];
char prev[2];

struct movement_data_priv
{
	int speed = 0;
	int angle = 0;

	void refresh(int speed_modifier, int diff_r_minus_l)
	{
		speed = (speed_modifier >= 0) ? speed_modifier : -speed_modifier;
		
		if (diff_r_minus_l > 0) //skrece ulijevo
		{
			angle = (speed_modifier >= 0) ? ANGLE_LEFT : ANGLE_LEFT_BACKWARDS;
		}
		else if (diff_r_minus_l < 0) // skrece udesno
		{
			angle = (speed_modifier >= 0) ? ANGLE_RIGHT : ANGLE_RIGHT_BACKWARDS;
		}
		else // left i right motor je isti speed == ravno
		{
			angle = (speed_modifier >= 0) ? ANGLE_STRAIGHT : ANGLE_BACKWARDS;
		}
	}
} dat;

void setup()
{
	// U konstruktoru OBAVEZNO dodijelite robotu ime kako piše na njegovoj naljepnici!!
	// Na taj ćemo način smo sigurni da se nikoja dva robota ne zopvu istim imenom
	// i moći ćemo s njima uspostaviti Bluetooth komunikaciju
	robot = new RobotSoccer((char *)"My robot"); // RobotSoccer, RobotMaze, RobotMin, RobotSoccer, or Your custom robot. "My robot" is Bluetooth name.
	robot->print("Start.\n\rOvo je naš prvi program za robota\n\r");
	strcpy(msg_private, "x");
}

void loop()
{
	robot->refresh();
}

/*
Funkcija članica loop pojedinih vrsta robota se izvršava kod svakog prolaska kroz petlju
Pokreće se opisivanjem loo u serijskom monitoru na računalu ili mobitelu
U tu svrhu je korisno imati računalo s Bluetooth vezom.
*/

/*
Definicija funkcije loop se izvorno nalazila u datoteci mrm-robot-line.cpp,
ali sam je premjestio ovdje kako ne bismo na radionici
morali mijenjati datoteku mrm-robot-line.cppp već samo main.cpp
(na radionici ćemo koristiti RobotSoccer)
*/

// start (loo)
void RobotSoccer::loop() 
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "S");
	//display(msg_private);

	if (strcmp(prev, "S") == 0)
	{
		dat.refresh(speed_modifier, 0);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		speed_modifier = MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - balanced (lo0)
void RobotSoccer::loop0() 
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "A");
	//display(msg_private);

	if (strcmp(prev, "A") == 0)
	{
		dat.refresh(speed_modifier, 0);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - left (lo1)
void RobotSoccer::loop1()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "C");
	//display(msg_private);

	if (strcmp(prev, "C") == 0)
	{
		if (speed_modifier == 0) // non-strafe
		{
			go(0, ANGLE_STRAIGHT, ROTATE_LEFT); // motor speed 0 za rotaciju na mjestu
		}
		else
		{
			dat.refresh(speed_modifier, rightmotor - leftmotor);
			go(dat.speed, dat.angle, STRAFE);
		}
	}
	else
	{
		leftmotor = speed_modifier / TILT_MODIFIER;
		rightmotor = speed_modifier * TILT_MODIFIER * SPEED_BEND_BOOST;

		if (speed_modifier == 0) // manevar
		{
			leftmotor = 0;
			rightmotor = last_direction * (MIN_SPEED_THRESHOLD + TIGHT_BEND_BOOST);
		}
		else
		{
			if (-MIN_SPEED_THRESHOLD <= leftmotor && leftmotor <= MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}

			if (rightmotor < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < rightmotor)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
		}
	}
}

// forward (lo2)
void RobotSoccer::loop2()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "D");
	//display(msg_private);

	if (strcmp(prev, "D") == 0)
	{
		dat.refresh(speed_modifier, rightmotor - leftmotor);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		speed_modifier = MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - right (lo3)
void RobotSoccer::loop3()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "E");
	//display(msg_private);

	if (strcmp(prev, "E") == 0)
	{
		if (speed_modifier == 0)
		{
			go(0, ANGLE_STRAIGHT, ROTATE_RIGHT);
		}
		else
		{
			dat.refresh(speed_modifier, rightmotor - leftmotor);
			go(dat.speed, dat.angle, STRAFE);
		}
	}
	else
	{
		rightmotor = speed_modifier / TILT_MODIFIER;
		leftmotor = speed_modifier * TILT_MODIFIER * SPEED_BEND_BOOST;

		if (speed_modifier == 0) // manevar
		{
			rightmotor = 0;
			leftmotor = last_direction * (MIN_SPEED_THRESHOLD + TIGHT_BEND_BOOST);
		}
		else
		{
			if (-MIN_SPEED_THRESHOLD <= rightmotor && rightmotor <= MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}

			if (leftmotor < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < leftmotor)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
		}
	}
}

// slower (lo4)
void RobotSoccer::loop4()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "F");
	//display(msg_private);

	if (strcmp(prev, "F") == 0)
	{
		dat.refresh(speed_modifier, rightmotor - leftmotor);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		float left;
		float right;

		left = (float)leftmotor / speed_modifier;
		right = (float)rightmotor / speed_modifier;
		speed_modifier -= (speed_modifier >= 0 ? 1 : -1) * SPEED_STEP;

		if (-MIN_SPEED_THRESHOLD < speed_modifier && speed_modifier < MIN_SPEED_THRESHOLD) 
		{
			last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
			leftmotor = rightmotor = speed_modifier = 0;
		}
		else
		{
			left *= speed_modifier;
			right *= speed_modifier;

			if (-MIN_SPEED_THRESHOLD < left && left < MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (left < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < left)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				leftmotor = left;
			}

			if (-MIN_SPEED_THRESHOLD < right && right < MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (right < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < right)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				rightmotor = right;
			}
		}
	}
}

// stop (lo5) -- postoji i BT komanda 'x', ovo je "soft stop" jer zelimo da se moze okretati oko sebe kad stane
void RobotSoccer::loop5()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "Q");
	//display(msg_private);

	if (strcmp(prev, "Q") != 0) // DRUGACIJE, PAZI
	{
		last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
		speed_modifier = 0;
		leftmotor = rightmotor = speed_modifier;
		go(speed_modifier, ANGLE_STRAIGHT, STRAFE);
	}
	// else - do nothing! ili stoji ili je spreman za manevar s tiltanjem
}

// faster (lo6)
void RobotSoccer::loop6()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "G");
	//display(msg_private);

	if (strcmp(prev, "G") == 0)
	{
		dat.refresh(speed_modifier, rightmotor - leftmotor);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		if (speed_modifier == 0)
		{
			speed_modifier = last_direction * MIN_SPEED_THRESHOLD;
			leftmotor = rightmotor = speed_modifier;
			return;
		}

		float left;
		float right;

		left = (float)leftmotor / speed_modifier;
		right = (float)rightmotor / speed_modifier;
		speed_modifier += (speed_modifier >= 0 ? 1 : -1) * SPEED_STEP;

		if (-MIN_SPEED_THRESHOLD < speed_modifier && speed_modifier < MIN_SPEED_THRESHOLD) 
		{
			last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
			leftmotor = rightmotor = speed_modifier = 0;
		}
		else
		{
			left *= speed_modifier;
			right *= speed_modifier;

			if (-MIN_SPEED_THRESHOLD < left && left < MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (left < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < left)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				leftmotor = left;
			}

			if (-MIN_SPEED_THRESHOLD < right && right < MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (right < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < right)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				rightmotor = right;
			}
		}
	}
}

// unused
void RobotSoccer::loop7()
{
}

// reverse (lo8)
void RobotSoccer::loop8()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "H");
	//display(msg_private);

	if (strcmp(prev, "H") == 0)
	{
		dat.refresh(speed_modifier, rightmotor - leftmotor);
		go(dat.speed, dat.angle, STRAFE);
	}
	else
	{
		speed_modifier = -MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// unused
void RobotSoccer::loop9()
{
}